package data

import (
	"fmt"
	"io"
	"net/http"
)

type ApiClient struct {
	Key      string
	Host     string
	Port     int
	Protocol string
}

func NewApiClient(key, host, protocol string, port int) *ApiClient {
	return &ApiClient{
		Key:      key,
		Host:     host,
		Port:     port,
		Protocol: protocol,
	}
}

func (a *ApiClient) Call(method, route string, requestBody io.Reader, useKey bool) (*http.Response, error) {
	var response *http.Response

	endpoint := fmt.Sprintf("%s://%s:%d%s",
		a.Protocol, a.Host, a.Port, route,
	)

	// Create client
	client := &http.Client{}

	// Create request
	request, err := http.NewRequest(method, endpoint, requestBody)
	if err != nil {
		return response, err
	}

	if useKey {
		if a.Key == "" {
			return response, fmt.Errorf("Call to API required a key but none was provided. See --help for instructions on providing an API key.")
		}

		request.Header.Add("Authorization", fmt.Sprintf("Bearer %s", a.Key))
	}

	if requestBody != nil {
		request.Header.Add("Content-Type", "application/json")
	}

	// Fetch Request
	response, err = client.Do(request)
	if err != nil {
		return response, err
	}

	return response, nil
}
